window.Gleam.initSearch([{"doc":"glitr","title":"glitr","content":"# glitr\r\n\r\nGleam typed routes library for type-safe fullstack development\r\n\r\n[![Package Version](https://img.shields.io/hexpm/v/glitr)](https://hex.pm/packages/glitr)\r\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glitr/)\r\n\r\n```sh\r\ngleam add glitr\r\n```\r\n```gleam\r\nimport glitr\r\nimport glitr/body\r\nimport glitr/route\r\nimport glitr/path\r\nimport glitr_lustre\r\nimport gleam/json\r\nimport gleam/dynamic\r\nimport gleam/http\r\n\r\npub type Todo {\r\n  Todo(id: String, title: String)\r\n}\r\n\r\nfn encoder(t: Todo) -&gt; json.Json {\r\n  json.object([\r\n    #(&quot;id&quot;, json.string(t.id)),\r\n    #(&quot;title&quot;, json.string(t.title)),\r\n  ])\r\n}\r\n\r\nfn decoder(\r\n  value: dynamic.Dynamic,\r\n) -&gt; Result(Todo, List(dynamic.DecodeError)) {\r\n  value\r\n  |&gt; dynamic.decode2(\r\n    Todo,\r\n    dynamic.field(&quot;id&quot;, dynamic.string),\r\n    dynamic.field(&quot;title&quot;, dynamic.string),\r\n  )\r\n}\r\n\r\npub fn main() {\r\n  let get_all_todos_route = route.new()\r\n    |&gt; with_path(path.static_path([&quot;todos&quot;]))\r\n    |&gt; with_response_body(body.json_body(encoder, decoder))\r\n\r\n  // This route can be shared and used by both frontend and backend applications\r\n\r\n  // Here is an example for a frontend lustre application using glitr_lustre\r\n  glitr_lustre.create_factory()\r\n    |&gt; glitr_lustre.with_port(2345)\r\n    |&gt; glitr_lustre.for_route(get_all_todos_route)\r\n    |&gt; glitr_lustre.with_path(Nil) // For now, this is necessary\r\n    |&gt; glitr_lustr.send(\r\n      fn(todos) { ServerSentTodos(todos) }, // ServerSentTodos is a Lustre message\r\n      fn(_err) { effect.none() } // This is for error handling\r\n    )\r\n}\r\n```\r\n\r\nFurther documentation can be found at &lt;https://hexdocs.pm/glitr&gt;.\r\n\r\n## Development\r\n\r\n```sh\r\ngleam run   # Run the project\r\ngleam test  # Run the tests\r\n```\r\n","url":"index.html"},{"doc":"glitr","title":"RouteOptions","content":"pub type RouteOptions(p, q, b) {\n  RouteOptions(path: p, query: q, body: b)\n}\n The options required to build a request from a Route\nRouteOptions(path: p, query: q, body: b)\n\n\nSynonyms:\nglitr.RouteOptions\nglitr RouteOptions","url":"glitr.html#RouteOptions"},{"doc":"glitr","title":"glitr","content":"","url":"glitr.html"},{"doc":"glitr/error","title":"GlitrError","content":"pub type GlitrError {\n  RouteError(msg: String)\n  JsonDecodeError(err: json.DecodeError)\n}\n Errors that can occur in Glitr code\nRouteError(msg: String)\n\n\nJsonDecodeError(err: json.DecodeError)\n\n\nSynonyms:\nglitr/error.GlitrError\nglitr/error GlitrError","url":"glitr/error.html#GlitrError"},{"doc":"glitr/error","title":"glitr/error","content":"","url":"glitr/error.html"},{"doc":"glitr/body","title":"BodyConverter","content":"pub type BodyConverter(body_type) {\n  BodyConverter(\n    encoder: fn(body_type) -&gt; string_builder.StringBuilder,\n    decoder: fn(String) -&gt; Result(body_type, error.GlitrError),\n  )\n}\n A wrapper for an encoder and a decoder to convert from request/response data to and from a Gleam type\nBodyConverter(\n  encoder: fn(body_type) -&gt; string_builder.StringBuilder,\n  decoder: fn(String) -&gt; Result(body_type, error.GlitrError),\n)\n\n\nSynonyms:\nglitr/body.BodyConverter\nglitr/body BodyConverter","url":"glitr/body.html#BodyConverter"},{"doc":"glitr/body","title":"BodyType","content":"pub type BodyType {\n  EmptyBody\n  StringBody\n  JsonBody\n}\n The type of body that can be expected from a Route\nEmptyBody\n\n\nStringBody\n\n\nJsonBody\n\n\nSynonyms:\nglitr/body.BodyType\nglitr/body BodyType","url":"glitr/body.html#BodyType"},{"doc":"glitr/body","title":"RouteBody","content":"pub opaque type RouteBody(body_type)\n The body type of a Route\n\nSynonyms:\nglitr/body.RouteBody\nglitr/body RouteBody","url":"glitr/body.html#RouteBody"},{"doc":"glitr/body","title":"decode","content":"pub fn decode(\n  body: RouteBody(a),\n  value: String,\n) -&gt; Result(a, GlitrError)\n Decode a value using the RouteBody&#39;s decoder from a String\nSynonyms:\nglitr/body.decode\nglitr/body decode","url":"glitr/body.html#decode"},{"doc":"glitr/body","title":"empty_body","content":"pub fn empty_body() -&gt; RouteBody(Nil)\n Create a RouteBody that will be empty\nSynonyms:\nglitr/body.empty_body\nglitr/body empty_body","url":"glitr/body.html#empty_body"},{"doc":"glitr/body","title":"encode","content":"pub fn encode(body: RouteBody(a), value: a) -&gt; StringBuilder\n Encode a value using the RouteBody&#39;s encoder into a StringBuilder\nSynonyms:\nglitr/body.encode\nglitr/body encode","url":"glitr/body.html#encode"},{"doc":"glitr/body","title":"get_type","content":"pub fn get_type(body: RouteBody(a)) -&gt; BodyType\n Return the BodyType of a RouteBody\nSynonyms:\nglitr/body.get_type\nglitr/body get_type","url":"glitr/body.html#get_type"},{"doc":"glitr/body","title":"json_body","content":"pub fn json_body(\n  encoder: fn(a) -&gt; Json,\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; RouteBody(a)\n Create a RouteBody that will be converted from/to json\nSynonyms:\nglitr/body.json_body\nglitr/body json_body","url":"glitr/body.html#json_body"},{"doc":"glitr/body","title":"string_body","content":"pub fn string_body(converter: BodyConverter(a)) -&gt; RouteBody(a)\n Create a RouteBody that will be converted from/to a string\nSynonyms:\nglitr/body.string_body\nglitr/body string_body","url":"glitr/body.html#string_body"},{"doc":"glitr/body","title":"glitr/body","content":" This module exports types and functions related to the body of Routes","url":"glitr/body.html"},{"doc":"glitr/path","title":"PathConverter","content":"pub type PathConverter(path_type) {\n  PathConverter(\n    encoder: fn(path_type) -&gt; List(String),\n    decoder: fn(List(String)) -&gt; Result(path_type, Nil),\n  )\n}\n A wrapper for an encoder and a decoder to convert from request/response path to and from a Gleam type  \n The decoder should return Error(Nil) if the input path doesn&#39;t correspond to the expected pattern\nPathConverter(\n  encoder: fn(path_type) -&gt; List(String),\n  decoder: fn(List(String)) -&gt; Result(path_type, Nil),\n)\n\n\nSynonyms:\nglitr/path.PathConverter\nglitr/path PathConverter","url":"glitr/path.html#PathConverter"},{"doc":"glitr/path","title":"PathType","content":"pub type PathType {\n  StaticPath\n  ComplexPath\n}\n The type of path that can be expected from a Route\nStaticPath\n\n\nComplexPath\n\n\nSynonyms:\nglitr/path.PathType\nglitr/path PathType","url":"glitr/path.html#PathType"},{"doc":"glitr/path","title":"RoutePath","content":"pub opaque type RoutePath(path_type)\n The path type of a Route\n\nSynonyms:\nglitr/path.RoutePath\nglitr/path RoutePath","url":"glitr/path.html#RoutePath"},{"doc":"glitr/path","title":"complex_path","content":"pub fn complex_path(converter: PathConverter(a)) -&gt; RoutePath(a)\n Create a more complex RoutePath from a custom converter\nSynonyms:\nglitr/path.complex_path\nglitr/path complex_path","url":"glitr/path.html#complex_path"},{"doc":"glitr/path","title":"decode","content":"pub fn decode(\n  path: RoutePath(a),\n  value: List(String),\n) -&gt; Result(a, Nil)\n Decode a value using the RoutePath&#39;s decoder from path segments\nSynonyms:\nglitr/path.decode\nglitr/path decode","url":"glitr/path.html#decode"},{"doc":"glitr/path","title":"encode","content":"pub fn encode(path: RoutePath(a), value: a) -&gt; List(String)\n Encode a value using the RoutePath&#39;s encoder into path segments\nSynonyms:\nglitr/path.encode\nglitr/path encode","url":"glitr/path.html#encode"},{"doc":"glitr/path","title":"get_type","content":"pub fn get_type(path: RoutePath(a)) -&gt; PathType\n Return the PathType of a RoutePath\nSynonyms:\nglitr/path.get_type\nglitr/path get_type","url":"glitr/path.html#get_type"},{"doc":"glitr/path","title":"id_path","content":"pub fn id_path(root: List(String)) -&gt; RoutePath(String)\n Create a RoutePath that will match path of the type `/x/y/z/:id`\n The id has to be the last segment of the path\nSynonyms:\nglitr/path.id_path\nglitr/path id_path","url":"glitr/path.html#id_path"},{"doc":"glitr/path","title":"static_path","content":"pub fn static_path(root: List(String)) -&gt; RoutePath(Nil)\n Create a RoutePath that will solely match a static path\nSynonyms:\nglitr/path.static_path\nglitr/path static_path","url":"glitr/path.html#static_path"},{"doc":"glitr/path","title":"glitr/path","content":" This module exports types and functions related to the path of Routes","url":"glitr/path.html"},{"doc":"glitr/query","title":"QueryConverter","content":"pub type QueryConverter(query_type) {\n  QueryConverter(\n    encoder: fn(query_type) -&gt; List(#(String, String)),\n    decoder: fn(List(#(String, String))) -&gt;\n      Result(query_type, Nil),\n  )\n}\n A wrapper for an encoder and a decoder to convert from request/response query to and from a Gleam type\nQueryConverter(\n  encoder: fn(query_type) -&gt; List(#(String, String)),\n  decoder: fn(List(#(String, String))) -&gt; Result(query_type, Nil),\n)\n\n\nSynonyms:\nglitr/query.QueryConverter\nglitr/query QueryConverter","url":"glitr/query.html#QueryConverter"},{"doc":"glitr/query","title":"QueryType","content":"pub type QueryType {\n  EmptyQuery\n  ComplexQuery\n}\n The type of query that can be expected from a Route\nEmptyQuery\n\n\nComplexQuery\n\n\nSynonyms:\nglitr/query.QueryType\nglitr/query QueryType","url":"glitr/query.html#QueryType"},{"doc":"glitr/query","title":"RouteQuery","content":"pub opaque type RouteQuery(query_type)\n The query type of a Route\n\nSynonyms:\nglitr/query.RouteQuery\nglitr/query RouteQuery","url":"glitr/query.html#RouteQuery"},{"doc":"glitr/query","title":"complex_query","content":"pub fn complex_query(\n  converter: QueryConverter(a),\n) -&gt; RouteQuery(a)\n Create a more complex RouteQuery from a custom converter\nSynonyms:\nglitr/query.complex_query\nglitr/query complex_query","url":"glitr/query.html#complex_query"},{"doc":"glitr/query","title":"decode","content":"pub fn decode(\n  query: RouteQuery(a),\n  value: List(#(String, String)),\n) -&gt; Result(a, Nil)\n Decode a value using the RouteQuery&#39;s decoder from a query\nSynonyms:\nglitr/query.decode\nglitr/query decode","url":"glitr/query.html#decode"},{"doc":"glitr/query","title":"empty_query","content":"pub fn empty_query() -&gt; RouteQuery(Nil)\n Create a RouteQuery that convert from/to an empty query\nSynonyms:\nglitr/query.empty_query\nglitr/query empty_query","url":"glitr/query.html#empty_query"},{"doc":"glitr/query","title":"encode","content":"pub fn encode(\n  query: RouteQuery(a),\n  value: a,\n) -&gt; List(#(String, String))\n Encode a value using the RouteQuery&#39;s encoder into a query\nSynonyms:\nglitr/query.encode\nglitr/query encode","url":"glitr/query.html#encode"},{"doc":"glitr/query","title":"get_type","content":"pub fn get_type(query: RouteQuery(a)) -&gt; QueryType\n Return the QueryType of a RouteQuery\nSynonyms:\nglitr/query.get_type\nglitr/query get_type","url":"glitr/query.html#get_type"},{"doc":"glitr/query","title":"glitr/query","content":" This module exports types and functions related to the query of Routes","url":"glitr/query.html"},{"doc":"glitr/route","title":"Route","content":"pub type Route(\n  path_type,\n  query_type,\n  req_body_type,\n  res_body_type,\n) {\n  Route(\n    method: http.Method,\n    path: path.RoutePath(path_type),\n    query: query.RouteQuery(query_type),\n    req_body: body.RouteBody(req_body_type),\n    res_body: body.RouteBody(res_body_type),\n  )\n}\n The Route type  \n Routes should be built using the builder functions\nRoute(\n  method: http.Method,\n  path: path.RoutePath(path_type),\n  query: query.RouteQuery(query_type),\n  req_body: body.RouteBody(req_body_type),\n  res_body: body.RouteBody(res_body_type),\n)\n\n\nSynonyms:\nglitr/route.Route\nglitr/route Route","url":"glitr/route.html#Route"},{"doc":"glitr/route","title":"new","content":"pub fn new() -&gt; Route(Nil, Nil, Nil, Nil)\n Create a new Route  \n By default the method is GET and the path, query and bodies are empty\nSynonyms:\nglitr/route.new\nglitr/route new","url":"glitr/route.html#new"},{"doc":"glitr/route","title":"with_method","content":"pub fn with_method(\n  route: Route(a, b, c, d),\n  method: Method,\n) -&gt; Route(a, b, c, d)\n Change the method of a Route\nSynonyms:\nglitr/route.with_method\nglitr/route with_method","url":"glitr/route.html#with_method"},{"doc":"glitr/route","title":"with_path","content":"pub fn with_path(\n  route: Route(a, b, c, d),\n  path: RoutePath(e),\n) -&gt; Route(e, b, c, d)\n Change the path of a Route\nSynonyms:\nglitr/route.with_path\nglitr/route with_path","url":"glitr/route.html#with_path"},{"doc":"glitr/route","title":"with_query","content":"pub fn with_query(\n  route: Route(a, b, c, d),\n  query: RouteQuery(e),\n) -&gt; Route(a, e, c, d)\n Change the query of a Route\nSynonyms:\nglitr/route.with_query\nglitr/route with_query","url":"glitr/route.html#with_query"},{"doc":"glitr/route","title":"with_request_body","content":"pub fn with_request_body(\n  route: Route(a, b, c, d),\n  req_body: RouteBody(e),\n) -&gt; Route(a, b, e, d)\n Change the request body of a Route\nSynonyms:\nglitr/route.with_request_body\nglitr/route with_request_body","url":"glitr/route.html#with_request_body"},{"doc":"glitr/route","title":"with_response_body","content":"pub fn with_response_body(\n  route: Route(a, b, c, d),\n  res_body: RouteBody(e),\n) -&gt; Route(a, b, c, e)\n Change the response body of a Route\nSynonyms:\nglitr/route.with_response_body\nglitr/route with_response_body","url":"glitr/route.html#with_response_body"},{"doc":"glitr/route","title":"glitr/route","content":" This module exports the Route type and builder functions.  \n Routes correspond to endpoints on a backend that will be called from a frontend.\n They are defined by a method, a path, a query and request/response bodies.","url":"glitr/route.html"},{"doc":"glitr/service","title":"RouteService","content":"pub type RouteService(base_type, upsert_type) {\n  RouteService(\n    root_path: List(String),\n    base: Option(\n      #(\n        fn(base_type) -&gt; json.Json,\n        fn(dynamic.Dynamic) -&gt;\n          Result(base_type, List(dynamic.DecodeError)),\n      ),\n    ),\n    upsert: Option(\n      #(\n        fn(upsert_type) -&gt; json.Json,\n        fn(dynamic.Dynamic) -&gt;\n          Result(upsert_type, List(dynamic.DecodeError)),\n      ),\n    ),\n  )\n}\n The RouteService type  \n Contains the data necessary to build the CRUD routes.\n Note that all data transmission will be done via JSON objects.\nRouteService(\n  root_path: List(String),\n  base: Option(\n    #(\n      fn(base_type) -&gt; json.Json,\n      fn(dynamic.Dynamic) -&gt;\n        Result(base_type, List(dynamic.DecodeError)),\n    ),\n  ),\n  upsert: Option(\n    #(\n      fn(upsert_type) -&gt; json.Json,\n      fn(dynamic.Dynamic) -&gt;\n        Result(upsert_type, List(dynamic.DecodeError)),\n    ),\n  ),\n)\n\n\nSynonyms:\nglitr/service.RouteService\nglitr/service RouteService","url":"glitr/service.html#RouteService"},{"doc":"glitr/service","title":"create_route","content":"pub fn create_route(\n  service: RouteService(a, b),\n) -&gt; Result(Route(Nil, Nil, b, a), GlitrError)\n Generate a create route associated with a service\nSynonyms:\nglitr/service.create_route\nglitr/service create_route","url":"glitr/service.html#create_route"},{"doc":"glitr/service","title":"delete_route","content":"pub fn delete_route(\n  service: RouteService(a, b),\n) -&gt; Result(Route(String, Nil, Nil, String), GlitrError)\n Generate a delete route associated with a service  \n Note that the return is the id of the deleted instance\nSynonyms:\nglitr/service.delete_route\nglitr/service delete_route","url":"glitr/service.html#delete_route"},{"doc":"glitr/service","title":"get_all_route","content":"pub fn get_all_route(\n  service: RouteService(a, b),\n) -&gt; Result(Route(Nil, Nil, Nil, List(a)), GlitrError)\n Generate a get-all route associated with a service\nSynonyms:\nglitr/service.get_all_route\nglitr/service get_all_route","url":"glitr/service.html#get_all_route"},{"doc":"glitr/service","title":"get_route","content":"pub fn get_route(\n  service: RouteService(a, b),\n) -&gt; Result(Route(String, Nil, Nil, a), GlitrError)\n Generate a get route associated with a service\nSynonyms:\nglitr/service.get_route\nglitr/service get_route","url":"glitr/service.html#get_route"},{"doc":"glitr/service","title":"new","content":"pub fn new() -&gt; RouteService(a, b)\n Create a new empty service  \n The base and upsert types will have to be specified !\nSynonyms:\nglitr/service.new\nglitr/service new","url":"glitr/service.html#new"},{"doc":"glitr/service","title":"update_route","content":"pub fn update_route(\n  service: RouteService(a, b),\n) -&gt; Result(Route(String, Nil, b, a), GlitrError)\n Generate a update route associated with a service\nSynonyms:\nglitr/service.update_route\nglitr/service update_route","url":"glitr/service.html#update_route"},{"doc":"glitr/service","title":"with_base_type","content":"pub fn with_base_type(\n  service: RouteService(a, b),\n  base_encoder: fn(c) -&gt; Json,\n  base_decoder: fn(Dynamic) -&gt; Result(c, List(DecodeError)),\n) -&gt; RouteService(c, b)\n Specify the base type of a service by providing a JSON encoder &amp; decoder  \n The base type of a service represent the type of object your service is associated with\nSynonyms:\nglitr/service.with_base_type\nglitr/service with_base_type","url":"glitr/service.html#with_base_type"},{"doc":"glitr/service","title":"with_root_path","content":"pub fn with_root_path(\n  service: RouteService(a, b),\n  root_path: List(String),\n) -&gt; RouteService(a, b)\n Change the root path of a service\nSynonyms:\nglitr/service.with_root_path\nglitr/service with_root_path","url":"glitr/service.html#with_root_path"},{"doc":"glitr/service","title":"with_upsert_type","content":"pub fn with_upsert_type(\n  service: RouteService(a, b),\n  upsert_encoder: fn(c) -&gt; Json,\n  upsert_decoder: fn(Dynamic) -&gt; Result(c, List(DecodeError)),\n) -&gt; RouteService(a, c)\n Specify the upsert type of a service by providing a JSON encoder &amp; decoder  \n The upsert type of a service represent the type used to create or update objects of your service\nSynonyms:\nglitr/service.with_upsert_type\nglitr/service with_upsert_type","url":"glitr/service.html#with_upsert_type"},{"doc":"glitr/service","title":"glitr/service","content":" This module helps you create standard CRUD services routes","url":"glitr/service.html"},{"doc":"glitr/utils","title":"to_request","content":"pub fn to_request(\n  route: Route(a, b, c, d),\n  path: a,\n  query: b,\n  body: c,\n) -&gt; Request(String)\n Converts a Route to a HTTP request by providing the path, query and request body data.  \n This function is actually not used in the library, it stays there in case one needs it.\nSynonyms:\nglitr/utils.to_request\nglitr/utils to_request","url":"glitr/utils.html#to_request"},{"doc":"glitr/utils","title":"glitr/utils","content":"","url":"glitr/utils.html"}]);